@@ -1,9 +1,9 @@
 # orm/state.py
-# Copyright (C) 2005-2021 the SQLAlchemy authors and contributors
+# Copyright (C) 2005-2023 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
-# the MIT License: http://www.opensource.org/licenses/mit-license.php
+# the MIT License: https://www.opensource.org/licenses/mit-license.php
 
 """Defines instrumentation of instances.
 
@@ -12,13 +12,28 @@ defines a large part of the ORM's interactivity.
 
 """
 
+from __future__ import annotations
+
+from typing import Any
+from typing import Callable
+from typing import Dict
+from typing import Generic
+from typing import Iterable
+from typing import Optional
+from typing import Set
+from typing import Tuple
+from typing import TYPE_CHECKING
+from typing import Union
 import weakref
 
 from . import base
 from . import exc as orm_exc
 from . import interfaces
+from ._typing import _O
+from ._typing import is_collection_impl
 from .base import ATTR_WAS_SET
 from .base import INIT_OK
+from .base import LoaderCallableStatus
 from .base import NEVER_SET
 from .base import NO_VALUE
 from .base import PASSIVE_NO_INITIALIZE
@@ -29,17 +44,62 @@ from .path_registry import PathRegistry
 from .. import exc as sa_exc
 from .. import inspection
 from .. import util
+from ..util.typing import Literal
+from ..util.typing import Protocol
+
+if TYPE_CHECKING:
+    from ._typing import _IdentityKeyType
+    from ._typing import _InstanceDict
+    from ._typing import _LoaderCallable
+    from .attributes import AttributeImpl
+    from .attributes import History
+    from .base import PassiveFlag
+    from .collections import _AdaptedCollectionProtocol
+    from .identity import IdentityMap
+    from .instrumentation import ClassManager
+    from .interfaces import ORMOption
+    from .mapper import Mapper
+    from .session import Session
+    from ..engine import Row
+    from ..ext.asyncio.session import async_session as _async_provider
+    from ..ext.asyncio.session import AsyncSession
+
+if TYPE_CHECKING:
+    _sessions: weakref.WeakValueDictionary[int, Session]
+else:
+    # late-populated by session.py
+    _sessions = None
 
 
-# late-populated by session.py
-_sessions = None
+if not TYPE_CHECKING:
+    # optionally late-provided by sqlalchemy.ext.asyncio.session
 
-# optionally late-provided by sqlalchemy.ext.asyncio.session
-_async_provider = None
+    _async_provider = None  # noqa
+
+
+class _InstanceDictProto(Protocol):
+    def __call__(self) -> Optional[IdentityMap]:
+        ...
+
+
+class _InstallLoaderCallableProto(Protocol[_O]):
+    """used at result loading time to install a _LoaderCallable callable
+    upon a specific InstanceState, which will be used to populate an
+    attribute when that attribute is accessed.
+
+    Concrete examples are per-instance deferred column loaders and
+    relationship lazy loaders.
+
+    """
+
+    def __call__(
+        self, state: InstanceState[_O], dict_: _InstanceDict, row: Row[Any]
+    ) -> None:
+        ...
 
 
 @inspection._self_inspects
-class InstanceState(interfaces.InspectionAttrInfo):
+class InstanceState(interfaces.InspectionAttrInfo, Generic[_O]):
     """tracks state information at the instance level.
 
     The :class:`.InstanceState` is a key object used by the
@@ -58,30 +118,66 @@ class InstanceState(interfaces.InspectionAttrInfo):
 
         >>> from sqlalchemy import inspect
         >>> insp = inspect(some_mapped_object)
+        >>> insp.attrs.nickname.history
+        History(added=['new nickname'], unchanged=(), deleted=['nickname'])
 
     .. seealso::
 
-        :ref:`core_inspection_toplevel`
+        :ref:`orm_mapper_inspection_instancestate`
 
     """
 
-    session_id = None
-    key = None
-    runid = None
-    load_options = util.EMPTY_SET
-    load_path = PathRegistry.root
-    insert_order = None
-    _strong_obj = None
-    modified = False
-    expired = False
-    _deleted = False
-    _load_pending = False
-    _orphaned_outside_of_session = False
-    is_instance = True
-    identity_token = None
-    _last_known_values = ()
+    __slots__ = (
+        "__dict__",
+        "__weakref__",
+        "class_",
+        "manager",
+        "obj",
+        "committed_state",
+        "expired_attributes",
+    )
 
-    callables = ()
+    manager: ClassManager[_O]
+    session_id: Optional[int] = None
+    key: Optional[_IdentityKeyType[_O]] = None
+    runid: Optional[int] = None
+    load_options: Tuple[ORMOption, ...] = ()
+    load_path: PathRegistry = PathRegistry.root
+    insert_order: Optional[int] = None
+    _strong_obj: Optional[object] = None
+    obj: weakref.ref[_O]
+
+    committed_state: Dict[str, Any]
+
+    modified: bool = False
+    expired: bool = False
+    _deleted: bool = False
+    _load_pending: bool = False
+    _orphaned_outside_of_session: bool = False
+    is_instance: bool = True
+    identity_token: object = None
+    _last_known_values: Optional[Dict[str, Any]] = None
+
+    _instance_dict: _InstanceDictProto
+    """A weak reference, or in the default case a plain callable, that
+    returns a reference to the current :class:`.IdentityMap`, if any.
+
+    """
+    if not TYPE_CHECKING:
+
+        def _instance_dict(self):
+            """default 'weak reference' for _instance_dict"""
+            return None
+
+    expired_attributes: Set[str]
+    """The set of keys which are 'expired' to be loaded by
+       the manager's deferred scalar loader, assuming no pending
+       changes.
+
+       see also the ``unmodified`` collection which is intersected
+       against this set when a refresh operation occurs."""
+
+    callables: Dict[str, Callable[[InstanceState[_O], PassiveFlag], Any]]
     """A namespace where a per-state loader callable can be associated.
 
     In SQLAlchemy 1.0, this is only used for lazy loaders / deferred
@@ -93,23 +189,18 @@ class InstanceState(interfaces.InspectionAttrInfo):
 
     """
 
-    def __init__(self, obj, manager):
+    if not TYPE_CHECKING:
+        callables = util.EMPTY_DICT
+
+    def __init__(self, obj: _O, manager: ClassManager[_O]):
         self.class_ = obj.__class__
         self.manager = manager
         self.obj = weakref.ref(obj, self._cleanup)
         self.committed_state = {}
         self.expired_attributes = set()
 
-    expired_attributes = None
-    """The set of keys which are 'expired' to be loaded by
-       the manager's deferred scalar loader, assuming no pending
-       changes.
-
-       see also the ``unmodified`` collection which is intersected
-       against this set when a refresh operation occurs."""
-
     @util.memoized_property
-    def attrs(self):
+    def attrs(self) -> util.ReadOnlyProperties[AttributeState]:
         """Return a namespace representing each attribute on
         the mapped object, including its current value
         and history.
@@ -120,12 +211,12 @@ class InstanceState(interfaces.InspectionAttrInfo):
         since the last flush.
 
         """
-        return util.ImmutableProperties(
-            dict((key, AttributeState(self, key)) for key in self.manager)
+        return util.ReadOnlyProperties(
+            {key: AttributeState(self, key) for key in self.manager}
         )
 
     @property
-    def transient(self):
+    def transient(self) -> bool:
         """Return ``True`` if the object is :term:`transient`.
 
         .. seealso::
@@ -136,7 +227,7 @@ class InstanceState(interfaces.InspectionAttrInfo):
         return self.key is None and not self._attached
 
     @property
-    def pending(self):
+    def pending(self) -> bool:
         """Return ``True`` if the object is :term:`pending`.
 
 
@@ -148,7 +239,7 @@ class InstanceState(interfaces.InspectionAttrInfo):
         return self.key is None and self._attached
 
     @property
-    def deleted(self):
+    def deleted(self) -> bool:
         """Return ``True`` if the object is :term:`deleted`.
 
         An object that is in the deleted state is guaranteed to
@@ -178,7 +269,7 @@ class InstanceState(interfaces.InspectionAttrInfo):
         return self.key is not None and self._attached and self._deleted
 
     @property
-    def was_deleted(self):
+    def was_deleted(self) -> bool:
         """Return True if this object is or was previously in the
         "deleted" state and has not been reverted to persistent.
 
@@ -187,9 +278,6 @@ class InstanceState(interfaces.InspectionAttrInfo):
         or via transaction commit and enters the "detached" state,
         this flag will continue to report True.
 
-        .. versionadded:: 1.1 - added a local method form of
-           :func:`.orm.util.was_deleted`.
-
         .. seealso::
 
             :attr:`.InstanceState.deleted` - refers to the "deleted" state
@@ -202,19 +290,13 @@ class InstanceState(interfaces.InspectionAttrInfo):
         return self._deleted
 
     @property
-    def persistent(self):
+    def persistent(self) -> bool:
         """Return ``True`` if the object is :term:`persistent`.
 
         An object that is in the persistent state is guaranteed to
         be within the :attr:`.Session.identity_map` of its parent
         :class:`.Session`.
 
-        .. versionchanged:: 1.1 The :attr:`.InstanceState.persistent`
-           accessor no longer returns True for an object that was
-           "deleted" within a flush; use the :attr:`.InstanceState.deleted`
-           accessor to detect this state.   This allows the "persistent"
-           state to guarantee membership in the identity map.
-
         .. seealso::
 
             :ref:`session_object_states`
@@ -223,7 +305,7 @@ class InstanceState(interfaces.InspectionAttrInfo):
         return self.key is not None and self._attached and not self._deleted
 
     @property
-    def detached(self):
+    def detached(self) -> bool:
         """Return ``True`` if the object is :term:`detached`.
 
         .. seealso::
@@ -233,15 +315,15 @@ class InstanceState(interfaces.InspectionAttrInfo):
         """
         return self.key is not None and not self._attached
 
-    @property
+    @util.non_memoized_property
     @util.preload_module("sqlalchemy.orm.session")
-    def _attached(self):
+    def _attached(self) -> bool:
         return (
             self.session_id is not None
             and self.session_id in util.preloaded.orm_session._sessions
         )
 
-    def _track_last_known_value(self, key):
+    def _track_last_known_value(self, key: str) -> None:
         """Track the last known value of a particular key after expiration
         operations.
 
@@ -249,12 +331,14 @@ class InstanceState(interfaces.InspectionAttrInfo):
 
         """
 
-        if key not in self._last_known_values:
-            self._last_known_values = dict(self._last_known_values)
-            self._last_known_values[key] = NO_VALUE
+        lkv = self._last_known_values
+        if lkv is None:
+            self._last_known_values = lkv = {}
+        if key not in lkv:
+            lkv[key] = NO_VALUE
 
     @property
-    def session(self):
+    def session(self) -> Optional[Session]:
         """Return the owning :class:`.Session` for this instance,
         or ``None`` if none available.
 
@@ -278,7 +362,7 @@ class InstanceState(interfaces.InspectionAttrInfo):
         return None
 
     @property
-    def async_session(self):
+    def async_session(self) -> Optional[AsyncSession]:
         """Return the owning :class:`_asyncio.AsyncSession` for this instance,
         or ``None`` if none available.
 
@@ -306,13 +390,17 @@ class InstanceState(interfaces.InspectionAttrInfo):
             return None
 
     @property
-    def object(self):
+    def object(self) -> Optional[_O]:
         """Return the mapped object represented by this
-        :class:`.InstanceState`."""
+        :class:`.InstanceState`.
+
+        Returns None if the object has been garbage collected
+
+        """
         return self.obj()
 
     @property
-    def identity(self):
+    def identity(self) -> Optional[Tuple[Any, ...]]:
         """Return the mapped identity of the mapped object.
         This is the primary key identity as persisted by the ORM
         which can always be passed directly to
@@ -332,7 +420,7 @@ class InstanceState(interfaces.InspectionAttrInfo):
             return self.key[1]
 
     @property
-    def identity_key(self):
+    def identity_key(self) -> Optional[_IdentityKeyType[_O]]:
         """Return the identity key for the mapped object.
 
         This is the key used to locate the object within
@@ -341,29 +429,27 @@ class InstanceState(interfaces.InspectionAttrInfo):
 
 
         """
-        # TODO: just change .key to .identity_key across
-        # the board ?  probably
         return self.key
 
     @util.memoized_property
-    def parents(self):
+    def parents(self) -> Dict[int, Union[Literal[False], InstanceState[Any]]]:
         return {}
 
     @util.memoized_property
-    def _pending_mutations(self):
+    def _pending_mutations(self) -> Dict[str, PendingCollection]:
         return {}
 
     @util.memoized_property
-    def _empty_collections(self):
+    def _empty_collections(self) -> Dict[str, _AdaptedCollectionProtocol]:
         return {}
 
     @util.memoized_property
-    def mapper(self):
+    def mapper(self) -> Mapper[_O]:
         """Return the :class:`_orm.Mapper` used for this mapped object."""
         return self.manager.mapper
 
     @property
-    def has_identity(self):
+    def has_identity(self) -> bool:
         """Return ``True`` if this object has an identity key.
 
         This should always have the same value as the
@@ -373,7 +459,12 @@ class InstanceState(interfaces.InspectionAttrInfo):
         return bool(self.key)
 
     @classmethod
-    def _detach_states(self, states, session, to_transient=False):
+    def _detach_states(
+        self,
+        states: Iterable[InstanceState[_O]],
+        session: Session,
+        to_transient: bool = False,
+    ) -> None:
         persistent_to_detached = (
             session.dispatch.persistent_to_detached or None
         )
@@ -405,17 +496,17 @@ class InstanceState(interfaces.InspectionAttrInfo):
 
             state._strong_obj = None
 
-    def _detach(self, session=None):
+    def _detach(self, session: Optional[Session] = None) -> None:
         if session:
             InstanceState._detach_states([self], session)
         else:
             self.session_id = self._strong_obj = None
 
-    def _dispose(self):
+    def _dispose(self) -> None:
+        # used by the test suite, apparently
         self._detach()
-        del self.obj
 
-    def _cleanup(self, ref):
+    def _cleanup(self, ref: weakref.ref[_O]) -> None:
         """Weakref callback cleanup.
 
         This callable cleans out the state when it is being garbage
@@ -443,13 +534,9 @@ class InstanceState(interfaces.InspectionAttrInfo):
             # assert self not in instance_dict._modified
 
         self.session_id = self._strong_obj = None
-        del self.obj
-
-    def obj(self):
-        return None
 
     @property
-    def dict(self):
+    def dict(self) -> _InstanceDict:
         """Return the instance dict used by the object.
 
         Under normal circumstances, this is always synonymous
@@ -467,35 +554,39 @@ class InstanceState(interfaces.InspectionAttrInfo):
         else:
             return {}
 
-    def _initialize_instance(*mixed, **kwargs):
+    def _initialize_instance(*mixed: Any, **kwargs: Any) -> None:
         self, instance, args = mixed[0], mixed[1], mixed[2:]  # noqa
         manager = self.manager
 
         manager.dispatch.init(self, args, kwargs)
 
         try:
-            return manager.original_init(*mixed[1:], **kwargs)
+            manager.original_init(*mixed[1:], **kwargs)
         except:
             with util.safe_reraise():
                 manager.dispatch.init_failure(self, args, kwargs)
 
-    def get_history(self, key, passive):
+    def get_history(self, key: str, passive: PassiveFlag) -> History:
         return self.manager[key].impl.get_history(self, self.dict, passive)
 
-    def get_impl(self, key):
+    def get_impl(self, key: str) -> AttributeImpl:
         return self.manager[key].impl
 
-    def _get_pending_mutation(self, key):
+    def _get_pending_mutation(self, key: str) -> PendingCollection:
         if key not in self._pending_mutations:
             self._pending_mutations[key] = PendingCollection()
         return self._pending_mutations[key]
 
-    def __getstate__(self):
-        state_dict = {"instance": self.obj()}
+    def __getstate__(self) -> Dict[str, Any]:
+        state_dict = {
+            "instance": self.obj(),
+            "class_": self.class_,
+            "committed_state": self.committed_state,
+            "expired_attributes": self.expired_attributes,
+        }
         state_dict.update(
             (k, self.__dict__[k])
             for k in (
-                "committed_state",
                 "_pending_mutations",
                 "modified",
                 "expired",
@@ -516,21 +607,18 @@ class InstanceState(interfaces.InspectionAttrInfo):
 
         return state_dict
 
-    def __setstate__(self, state_dict):
+    def __setstate__(self, state_dict: Dict[str, Any]) -> None:
         inst = state_dict["instance"]
         if inst is not None:
             self.obj = weakref.ref(inst, self._cleanup)
             self.class_ = inst.__class__
         else:
-            # None being possible here generally new as of 0.7.4
-            # due to storage of state in "parents".  "class_"
-            # also new.
-            self.obj = None
+            self.obj = lambda: None  # type: ignore
             self.class_ = state_dict["class_"]
 
         self.committed_state = state_dict.get("committed_state", {})
-        self._pending_mutations = state_dict.get("_pending_mutations", {})
-        self.parents = state_dict.get("parents", {})
+        self._pending_mutations = state_dict.get("_pending_mutations", {})  # type: ignore  # noqa E501
+        self.parents = state_dict.get("parents", {})  # type: ignore
         self.modified = state_dict.get("modified", False)
         self.expired = state_dict.get("expired", False)
         if "info" in state_dict:
@@ -538,15 +626,7 @@ class InstanceState(interfaces.InspectionAttrInfo):
         if "callables" in state_dict:
             self.callables = state_dict["callables"]
 
-            try:
-                self.expired_attributes = state_dict["expired_attributes"]
-            except KeyError:
-                self.expired_attributes = set()
-                # 0.9 and earlier compat
-                for k in list(self.callables):
-                    if self.callables[k] is self:
-                        self.expired_attributes.add(k)
-                        del self.callables[k]
+            self.expired_attributes = state_dict["expired_attributes"]
         else:
             if "expired_attributes" in state_dict:
                 self.expired_attributes = state_dict["expired_attributes"]
@@ -561,57 +641,61 @@ class InstanceState(interfaces.InspectionAttrInfo):
             ]
         )
         if self.key:
-            try:
-                self.identity_token = self.key[2]
-            except IndexError:
-                # 1.1 and earlier compat before identity_token
-                assert len(self.key) == 2
-                self.key = self.key + (None,)
-                self.identity_token = None
+            self.identity_token = self.key[2]
 
         if "load_path" in state_dict:
             self.load_path = PathRegistry.deserialize(state_dict["load_path"])
 
         state_dict["manager"](self, inst, state_dict)
 
-    def _reset(self, dict_, key):
+    def _reset(self, dict_: _InstanceDict, key: str) -> None:
         """Remove the given attribute and any
         callables associated with it."""
 
         old = dict_.pop(key, None)
-        if old is not None and self.manager[key].impl.collection:
-            self.manager[key].impl._invalidate_collection(old)
+        manager_impl = self.manager[key].impl
+        if old is not None and is_collection_impl(manager_impl):
+            manager_impl._invalidate_collection(old)
         self.expired_attributes.discard(key)
         if self.callables:
             self.callables.pop(key, None)
 
-    def _copy_callables(self, from_):
+    def _copy_callables(self, from_: InstanceState[Any]) -> None:
         if "callables" in from_.__dict__:
             self.callables = dict(from_.callables)
 
     @classmethod
-    def _instance_level_callable_processor(cls, manager, fn, key):
+    def _instance_level_callable_processor(
+        cls, manager: ClassManager[_O], fn: _LoaderCallable, key: Any
+    ) -> _InstallLoaderCallableProto[_O]:
         impl = manager[key].impl
-        if impl.collection:
+        if is_collection_impl(impl):
+            fixed_impl = impl
 
-            def _set_callable(state, dict_, row):
+            def _set_callable(
+                state: InstanceState[_O], dict_: _InstanceDict, row: Row[Any]
+            ) -> None:
                 if "callables" not in state.__dict__:
                     state.callables = {}
                 old = dict_.pop(key, None)
                 if old is not None:
-                    impl._invalidate_collection(old)
+                    fixed_impl._invalidate_collection(old)
                 state.callables[key] = fn
 
         else:
 
-            def _set_callable(state, dict_, row):
+            def _set_callable(
+                state: InstanceState[_O], dict_: _InstanceDict, row: Row[Any]
+            ) -> None:
                 if "callables" not in state.__dict__:
                     state.callables = {}
                 state.callables[key] = fn
 
         return _set_callable
 
-    def _expire(self, dict_, modified_set):
+    def _expire(
+        self, dict_: _InstanceDict, modified_set: Set[InstanceState[Any]]
+    ) -> None:
         self.expired = True
         if self.modified:
             modified_set.discard(self)
@@ -651,7 +735,7 @@ class InstanceState(interfaces.InspectionAttrInfo):
 
         if self._last_known_values:
             self._last_known_values.update(
-                (k, dict_[k]) for k in self._last_known_values if k in dict_
+                {k: dict_[k] for k in self._last_known_values if k in dict_}
             )
 
         for key in self.manager._all_key_set.intersection(dict_):
@@ -659,7 +743,12 @@ class InstanceState(interfaces.InspectionAttrInfo):
 
         self.manager.dispatch.expire(self, None)
 
-    def _expire_attributes(self, dict_, attribute_names, no_loader=False):
+    def _expire_attributes(
+        self,
+        dict_: _InstanceDict,
+        attribute_names: Iterable[str],
+        no_loader: bool = False,
+    ) -> None:
         pending = self.__dict__.get("_pending_mutations", None)
 
         callables = self.callables
@@ -674,15 +763,12 @@ class InstanceState(interfaces.InspectionAttrInfo):
                 if callables and key in callables:
                     del callables[key]
             old = dict_.pop(key, NO_VALUE)
-            if impl.collection and old is not NO_VALUE:
+            if is_collection_impl(impl) and old is not NO_VALUE:
                 impl._invalidate_collection(old)
 
-            if (
-                self._last_known_values
-                and key in self._last_known_values
-                and old is not NO_VALUE
-            ):
-                self._last_known_values[key] = old
+            lkv = self._last_known_values
+            if lkv is not None and key in lkv and old is not NO_VALUE:
+                lkv[key] = old
 
             self.committed_state.pop(key, None)
             if pending:
@@ -690,7 +776,9 @@ class InstanceState(interfaces.InspectionAttrInfo):
 
         self.manager.dispatch.expire(self, attribute_names)
 
-    def _load_expired(self, state, passive):
+    def _load_expired(
+        self, state: InstanceState[_O], passive: PassiveFlag
+    ) -> LoaderCallableStatus:
         """__call__ allows the InstanceState to act as a deferred
         callable for loading expired attributes, which is also
         serializable (picklable).
@@ -718,12 +806,12 @@ class InstanceState(interfaces.InspectionAttrInfo):
         return ATTR_WAS_SET
 
     @property
-    def unmodified(self):
+    def unmodified(self) -> Set[str]:
         """Return the set of keys which have no uncommitted changes"""
 
         return set(self.manager).difference(self.committed_state)
 
-    def unmodified_intersection(self, keys):
+    def unmodified_intersection(self, keys: Iterable[str]) -> Set[str]:
         """Return self.unmodified.intersection(keys)."""
 
         return (
@@ -733,7 +821,7 @@ class InstanceState(interfaces.InspectionAttrInfo):
         )
 
     @property
-    def unloaded(self):
+    def unloaded(self) -> Set[str]:
         """Return the set of keys which do not have a loaded value.
 
         This includes expired attributes and any other attribute that
@@ -747,7 +835,7 @@ class InstanceState(interfaces.InspectionAttrInfo):
         )
 
     @property
-    def unloaded_expirable(self):
+    def unloaded_expirable(self) -> Set[str]:
         """Return the set of keys which do not have a loaded value.
 
         This includes expired attributes and any other attribute that
@@ -757,19 +845,21 @@ class InstanceState(interfaces.InspectionAttrInfo):
         return self.unloaded
 
     @property
-    def _unloaded_non_object(self):
+    def _unloaded_non_object(self) -> Set[str]:
         return self.unloaded.intersection(
             attr
             for attr in self.manager
             if self.manager[attr].impl.accepts_scalar_loader
         )
 
-    def _instance_dict(self):
-        return None
-
     def _modified_event(
-        self, dict_, attr, previous, collection=False, is_userland=False
-    ):
+        self,
+        dict_: _InstanceDict,
+        attr: Optional[AttributeImpl],
+        previous: Any,
+        collection: bool = False,
+        is_userland: bool = False,
+    ) -> None:
         if attr:
             if not attr.send_modified_events:
                 return
@@ -780,6 +870,8 @@ class InstanceState(interfaces.InspectionAttrInfo):
                 )
             if attr.key not in self.committed_state or is_userland:
                 if collection:
+                    if TYPE_CHECKING:
+                        assert is_collection_impl(attr)
                     if previous is NEVER_SET:
                         if attr.key in dict_:
                             previous = dict_[attr.key]
@@ -788,8 +880,9 @@ class InstanceState(interfaces.InspectionAttrInfo):
                         previous = attr.copy(previous)
                 self.committed_state[attr.key] = previous
 
-            if attr.key in self._last_known_values:
-                self._last_known_values[attr.key] = NO_VALUE
+            lkv = self._last_known_values
+            if lkv is not None and attr.key in lkv:
+                lkv[attr.key] = NO_VALUE
 
         # assert self._strong_obj is None or self.modified
 
@@ -821,7 +914,7 @@ class InstanceState(interfaces.InspectionAttrInfo):
                         pass
                     else:
                         if session._transaction is None:
-                            session._autobegin()
+                            session._autobegin_t()
 
             if inst is None and attr:
                 raise orm_exc.ObjectDereferencedError(
@@ -831,7 +924,7 @@ class InstanceState(interfaces.InspectionAttrInfo):
                     % (self.manager[attr.key], base.state_class_str(self))
                 )
 
-    def _commit(self, dict_, keys):
+    def _commit(self, dict_: _InstanceDict, keys: Iterable[str]) -> None:
         """Commit attributes.
 
         This is used by a partial-attribute load operation to mark committed
@@ -860,7 +953,9 @@ class InstanceState(interfaces.InspectionAttrInfo):
             ):
                 del self.callables[key]
 
-    def _commit_all(self, dict_, instance_dict=None):
+    def _commit_all(
+        self, dict_: _InstanceDict, instance_dict: Optional[IdentityMap] = None
+    ) -> None:
         """commit all attributes unconditionally.
 
         This is used after a flush() or a full load/refresh
@@ -879,7 +974,11 @@ class InstanceState(interfaces.InspectionAttrInfo):
         self._commit_all_states([(self, dict_)], instance_dict)
 
     @classmethod
-    def _commit_all_states(self, iter_, instance_dict=None):
+    def _commit_all_states(
+        self,
+        iter_: Iterable[Tuple[InstanceState[Any], _InstanceDict]],
+        instance_dict: Optional[IdentityMap] = None,
+    ) -> None:
         """Mass / highly inlined version of commit_all()."""
 
         for state, dict_ in iter_:
@@ -899,7 +998,7 @@ class InstanceState(interfaces.InspectionAttrInfo):
             state._strong_obj = None
 
 
-class AttributeState(object):
+class AttributeState:
     """Provide an inspection interface corresponding
     to a particular attribute on a particular mapped object.
 
@@ -914,12 +1013,17 @@ class AttributeState(object):
 
     """
 
-    def __init__(self, state, key):
+    __slots__ = ("state", "key")
+
+    state: InstanceState[Any]
+    key: str
+
+    def __init__(self, state: InstanceState[Any], key: str):
         self.state = state
         self.key = key
 
     @property
-    def loaded_value(self):
+    def loaded_value(self) -> Any:
         """The current value of this attribute as loaded from the database.
 
         If the value has not been loaded, or is otherwise not present
@@ -929,7 +1033,7 @@ class AttributeState(object):
         return self.state.dict.get(self.key, NO_VALUE)
 
     @property
-    def value(self):
+    def value(self) -> Any:
         """Return the value of this attribute.
 
         This operation is equivalent to accessing the object's
@@ -942,7 +1046,7 @@ class AttributeState(object):
         )
 
     @property
-    def history(self):
+    def history(self) -> History:
         """Return the current **pre-flush** change history for
         this attribute, via the :class:`.History` interface.
 
@@ -969,7 +1073,7 @@ class AttributeState(object):
         """
         return self.state.get_history(self.key, PASSIVE_NO_INITIALIZE)
 
-    def load_history(self):
+    def load_history(self) -> History:
         """Return the current **pre-flush** change history for
         this attribute, via the :class:`.History` interface.
 
@@ -991,13 +1095,11 @@ class AttributeState(object):
 
             :func:`.attributes.get_history` - underlying function
 
-        .. versionadded:: 0.9.0
-
         """
         return self.state.get_history(self.key, PASSIVE_OFF ^ INIT_OK)
 
 
-class PendingCollection(object):
+class PendingCollection:
     """A writable placeholder for an unloaded collection.
 
     Stores items appended to and removed from a collection that has not yet
@@ -1006,17 +1108,22 @@ class PendingCollection(object):
 
     """
 
-    def __init__(self):
+    __slots__ = ("deleted_items", "added_items")
+
+    deleted_items: util.IdentitySet
+    added_items: util.OrderedIdentitySet
+
+    def __init__(self) -> None:
         self.deleted_items = util.IdentitySet()
         self.added_items = util.OrderedIdentitySet()
 
-    def append(self, value):
+    def append(self, value: Any) -> None:
         if value in self.deleted_items:
             self.deleted_items.remove(value)
         else:
             self.added_items.add(value)
 
-    def remove(self, value):
+    def remove(self, value: Any) -> None:
         if value in self.added_items:
             self.added_items.remove(value)
         else:
