@@ -8,7 +8,10 @@ from sqlalchemy import column
 from sqlalchemy import create_engine
 from sqlalchemy import insert
 from sqlalchemy import Integer
+from sqlalchemy import MetaData
 from sqlalchemy import select
+from sqlalchemy import String
+from sqlalchemy import Table
 from sqlalchemy import table
 from sqlalchemy.ext.asyncio import AsyncConnection
 from sqlalchemy.ext.asyncio import AsyncSession
@@ -31,6 +34,14 @@ class User(Base):
     name: Mapped[str]
 
 
+t_user = Table(
+    "user",
+    MetaData(),
+    Column("id", Integer, primary_key=True),
+    Column("name", String),
+)
+
+
 e = create_engine("sqlite://")
 ae = create_async_engine("sqlite+aiosqlite://")
 
@@ -100,6 +111,11 @@ def t_result_ctxmanager() -> None:
         reveal_type(r4)
 
 
+def t_core_mappings() -> None:
+    r = connection.execute(select(t_user)).mappings().one()
+    r.get(t_user.c.id)
+
+
 def t_entity_varieties() -> None:
 
     a1 = aliased(User)
