@@ -170,6 +170,10 @@ class DefaultRequirements(SuiteRequirements):
     def comment_reflection(self):
         return only_on(["postgresql", "mysql", "mariadb", "oracle", "mssql"])
 
+    @property
+    def comment_reflection_full_unicode(self):
+        return only_on(["postgresql", "oracle", "mssql"])
+
     @property
     def constraint_comment_reflection(self):
         return only_on(["postgresql"])
@@ -1395,6 +1399,10 @@ class DefaultRequirements(SuiteRequirements):
             "postgresql+pg8000", "seems to work on pg14 only, not earlier?"
         )
 
+    @property
+    def float_or_double_precision_behaves_generically(self):
+        return skip_if(["oracle", "mysql", "mariadb"])
+
     @property
     def precision_generic_float_type(self):
         """target backend will return native floating point numbers with at
