@@ -1,5 +1,6 @@
 from itertools import zip_longest
 
+from sqlalchemy import bindparam
 from sqlalchemy import Column
 from sqlalchemy import Integer
 from sqlalchemy import MetaData
@@ -7,6 +8,7 @@ from sqlalchemy import select
 from sqlalchemy import String
 from sqlalchemy import Table
 from sqlalchemy import testing
+from sqlalchemy import TypeDecorator
 from sqlalchemy.sql import base as sql_base
 from sqlalchemy.sql import coercions
 from sqlalchemy.sql import column
@@ -18,6 +20,8 @@ from sqlalchemy.testing import assert_raises_message
 from sqlalchemy.testing import eq_
 from sqlalchemy.testing import expect_raises_message
 from sqlalchemy.testing import fixtures
+from sqlalchemy.testing import is_
+from sqlalchemy.testing import is_not_none
 
 
 class MiscTest(fixtures.TestBase):
@@ -41,6 +45,28 @@ class MiscTest(fixtures.TestBase):
 
         eq_(set(sql_util.find_tables(subset_select)), {common})
 
+    @testing.variation("has_cache_key", [True, False])
+    def test_get_embedded_bindparams(self, has_cache_key):
+        bp = bindparam("x")
+
+        if not has_cache_key:
+
+            class NotCacheable(TypeDecorator):
+                impl = String
+                cache_ok = False
+
+            stmt = select(column("q", NotCacheable())).where(column("y") == bp)
+
+        else:
+            stmt = select(column("q")).where(column("y") == bp)
+
+        eq_(stmt._get_embedded_bindparams(), [bp])
+
+        if not has_cache_key:
+            is_(stmt._generate_cache_key(), None)
+        else:
+            is_not_none(stmt._generate_cache_key())
+
     def test_find_tables_aliases(self):
         metadata = MetaData()
         common = Table(
